@@ -32,7 +32,7 @@ COMMENT_INDICATORS = ('"""', "'''", "'", '"', "#")
 
 def sort_code_string(
     code: str,
-    extension: str = "py",
+    extension: Optional[str] = None,
     config: Config = DEFAULT_CONFIG,
     file_path: Optional[Path] = None,
     disregard_skip: bool = False,
@@ -41,7 +41,7 @@ def sort_code_string(
     """Sorts any imports within the provided code string, returning a new string with them sorted.
 
     - **code**: The string of code with imports that need to be sorted.
-    - **extension**: The file extension that contains the code.
+    - **extension**: The file extension that contains imports. Defaults to filename extension or py.
     - **config**: The config object to use when sorting imports.
     - **file_path**: The disk location where the code string was pulled from.
     - **disregard_skip**: set to `True` if you want to ignore a skip set in config for this file.
@@ -65,7 +65,7 @@ def sort_code_string(
 def check_code_string(
     code: str,
     show_diff: bool = False,
-    extension: str = "py",
+    extension: Optional[str] = None,
     config: Config = DEFAULT_CONFIG,
     file_path: Optional[Path] = None,
     disregard_skip: bool = False,
@@ -76,7 +76,7 @@ def check_code_string(
 
     - **code**: The string of code with imports that need to be sorted.
     - **show_diff**: If `True` the changes that need to be done will be printed to stdout.
-    - **extension**: The file extension that contains the code.
+    - **extension**: The file extension that contains imports. Defaults to filename extension or py.
     - **config**: The config object to use when sorting imports.
     - **file_path**: The disk location where the code string was pulled from.
     - **disregard_skip**: set to `True` if you want to ignore a skip set in config for this file.
@@ -96,7 +96,7 @@ def check_code_string(
 def sort_stream(
     input_stream: TextIO,
     output_stream: TextIO,
-    extension: str = "py",
+    extension: Optional[str] = None,
     config: Config = DEFAULT_CONFIG,
     file_path: Optional[Path] = None,
     disregard_skip: bool = False,
@@ -107,7 +107,7 @@ def sort_stream(
 
     - **input_stream**: The stream of code with imports that need to be sorted.
     - **output_stream**: The stream where sorted imports should be written to.
-    - **extension**: The file extension that contains the code.
+    - **extension**: The file extension that contains imports. Defaults to filename extension or py.
     - **config**: The config object to use when sorting imports.
     - **file_path**: The disk location where the code string was pulled from.
     - **disregard_skip**: set to `True` if you want to ignore a skip set in config for this file.
@@ -133,7 +133,12 @@ def sort_stream(
             _internal_output = StringIO()
 
     try:
-        changed = _sort_imports(input_stream, _internal_output, extension=extension, config=config)
+        changed = _sort_imports(
+            input_stream,
+            _internal_output,
+            extension=extension or (file_path and file_path.suffix.lstrip(".")) or "py",
+            config=config,
+        )
     except FileSkipComment:
         raise FileSkipComment(content_source)
 
@@ -153,7 +158,7 @@ def sort_stream(
 def check_stream(
     input_stream: TextIO,
     show_diff: bool = False,
-    extension: str = "py",
+    extension: Optional[str] = None,
     config: Config = DEFAULT_CONFIG,
     file_path: Optional[Path] = None,
     disregard_skip: bool = False,
@@ -164,7 +169,7 @@ def check_stream(
 
     - **input_stream**: The stream of code with imports that need to be sorted.
     - **show_diff**: If `True` the changes that need to be done will be printed to stdout.
-    - **extension**: The file extension that contains the code.
+    - **extension**: The file extension that contains imports. Defaults to filename extension or py.
     - **config**: The config object to use when sorting imports.
     - **file_path**: The disk location where the code string was pulled from.
     - **disregard_skip**: set to `True` if you want to ignore a skip set in config for this file.
@@ -213,6 +218,7 @@ def check_file(
     config: Config = DEFAULT_CONFIG,
     file_path: Optional[Path] = None,
     disregard_skip: bool = True,
+    extension: Optional[str] = None,
     **config_kwargs,
 ) -> bool:
     """Checks any imports within the provided file, returning `False` if any unsorted or
@@ -220,17 +226,17 @@ def check_file(
 
     - **filename**: The name or Path of the file to check.
     - **show_diff**: If `True` the changes that need to be done will be printed to stdout.
-    - **extension**: The file extension that contains the code.
     - **config**: The config object to use when sorting imports.
     - **file_path**: The disk location where the code string was pulled from.
     - **disregard_skip**: set to `True` if you want to ignore a skip set in config for this file.
+    - **extension**: The file extension that contains imports. Defaults to filename extension or py.
     - ****config_kwargs**: Any config modifications.
     """
     with io.File.read(filename) as source_file:
         return check_stream(
             source_file.stream,
             show_diff=show_diff,
-            extension=source_file.extension or "py",
+            extension=extension,
             config=config,
             file_path=file_path or source_file.path,
             disregard_skip=disregard_skip,
@@ -240,7 +246,7 @@ def check_file(
 
 def sort_file(
     filename: Union[str, Path],
-    extension: str = "py",
+    extension: Optional[str] = None,
     config: Config = DEFAULT_CONFIG,
     file_path: Optional[Path] = None,
     disregard_skip: bool = True,
@@ -252,7 +258,7 @@ def sort_file(
     """Sorts and formats any groups of imports imports within the provided file or Path.
 
     - **filename**: The name or Path of the file to format.
-    - **extension**: The file extension that contains the code.
+    - **extension**: The file extension that contains imports. Defaults to filename extension or py.
     - **config**: The config object to use when sorting imports.
     - **file_path**: The disk location where the code string was pulled from.
     - **disregard_skip**: set to `True` if you want to ignore a skip set in config for this file.
@@ -271,6 +277,7 @@ def sort_file(
                     config=config,
                     file_path=file_path or source_file.path,
                     disregard_skip=disregard_skip,
+                    extension=extension,
                     **config_kwargs,
                 )
             else:
@@ -286,6 +293,7 @@ def sort_file(
                             config=config,
                             file_path=file_path or source_file.path,
                             disregard_skip=disregard_skip,
+                            extension=extension,
                             **config_kwargs,
                         )
                     if changed:
@@ -442,6 +450,13 @@ def _sort_imports(
                     isort_off = True
                 elif stripped_line == "# isort: split":
                     not_imports = True
+                elif not (stripped_line or contains_imports):
+                    if add_imports and not indent:
+                        import_section += line_separator.join(add_imports) + line_separator
+                        contains_imports = True
+                        add_imports = []
+                    else:
+                        not_imports = True
                 elif (
                     not stripped_line
                     or stripped_line.startswith("#")
@@ -494,6 +509,7 @@ def _sort_imports(
                     not_imports = True
 
         if not_imports:
+            raw_import_section: str = import_section
             if (
                 add_imports
                 and not in_top_comment
@@ -506,17 +522,20 @@ def _sort_imports(
                 add_imports = []
 
             if next_import_section and not import_section:  # pragma: no cover
-                import_section = next_import_section
+                raw_import_section = import_section = next_import_section
                 next_import_section = ""
 
             if import_section:
                 if add_imports and not indent:
-                    import_section += line_separator.join(add_imports) + line_separator
+                    import_section = (
+                        line_separator.join(add_imports) + line_separator + import_section
+                    )
                     contains_imports = True
                     add_imports = []
 
                 if not indent:
                     import_section += line
+                    raw_import_section += line
                 if not contains_imports:
                     output_stream.write(import_section)
                 else:
@@ -526,15 +545,17 @@ def _sort_imports(
                         line_separator
                     ).startswith(COMMENT_INDICATORS):
                         import_section = import_section.lstrip(line_separator)
+                        raw_import_section = raw_import_section.lstrip(line_separator)
                         first_import_section = False
 
-                    raw_import_section: str = import_section
                     if indent:
                         import_section = line_separator.join(
                             line.lstrip() for line in import_section.split(line_separator)
                         )
                         out_config = Config(
-                            config=config, line_length=max(config.line_length - len(indent), 0)
+                            config=config,
+                            line_length=max(config.line_length - len(indent), 0),
+                            lines_after_imports=1,
                         )
                     else:
                         out_config = config
@@ -545,31 +566,32 @@ def _sort_imports(
                         extension,
                         import_type="cimport" if cimports else "import",
                     )
-                    if indent:
-                        sorted_import_section = (
-                            leading_whitespace
-                            + textwrap.indent(sorted_import_section, indent).strip()
-                            + trailing_whitespace
-                        )
+                    if not (import_section.strip() and not sorted_import_section):
+                        if indent:
+                            sorted_import_section = (
+                                leading_whitespace
+                                + textwrap.indent(sorted_import_section, indent).strip()
+                                + trailing_whitespace
+                            )
 
-                    if not made_changes:
-                        if config.ignore_whitespace:
-                            compare_in = remove_whitespace(
-                                raw_import_section, line_separator=line_separator
-                            ).strip()
-                            compare_out = remove_whitespace(
-                                sorted_import_section, line_separator=line_separator
-                            ).strip()
-                        else:
-                            compare_in = raw_import_section.strip()
-                            compare_out = sorted_import_section.strip()
+                        if not made_changes:
+                            if config.ignore_whitespace:
+                                compare_in = remove_whitespace(
+                                    raw_import_section, line_separator=line_separator
+                                ).strip()
+                                compare_out = remove_whitespace(
+                                    sorted_import_section, line_separator=line_separator
+                                ).strip()
+                            else:
+                                compare_in = raw_import_section.strip()
+                                compare_out = sorted_import_section.strip()
 
-                        if compare_out != compare_in:
-                            made_changes = True
+                            if compare_out != compare_in:
+                                made_changes = True
 
-                    output_stream.write(sorted_import_section)
-                    if not line and not indent and next_import_section:
-                        output_stream.write(line_separator)
+                        output_stream.write(sorted_import_section)
+                        if not line and not indent and next_import_section:
+                            output_stream.write(line_separator)
 
                 if indent:
                     output_stream.write(line)
